// winThrottle - (c) 2002-2006 jeff leyda  jeff@silent.net 
// with overwhelimingly massive amounts of help from Ryan Geiss geiss.3@geisswerks.com 
//
// the executable and source code for winThrottle are free to use as you see fit, provided it is not used for
// financial gain.  It would be nice if you contacted me if you plan on using any of it.
//
// Thanks to Sander Aiaots for the inspiration to port my DOS program to a windows app.
// portions of the following code are his.
//
// winThrottle uses chunks of code from JetByte software (registry reader)
// as well as CPU load reporting code obtained from Dudi Avramov via the code project.
// http://www.codeproject.com/system/cpuusage.asp
// plus the tray icon code from Muneeb Ahmed Awan from the code project pages
// http://www.codeproject.com/shell/CTrayIcon.asp
// Respective copywrites are theirs.  They rock, and there is no way winThrottle would exist without them.
//
// Huge thanks to Ryan Geiss as mentioned above, who took winThrottle to whole new levels of functionality.
// 
// When compiling, I get 8 warnings from the included code.  I didn't investigate fixing them, since none of them are mine. ;)

#include "stdafx.h"
#include "Throttle.h"
#include "ThrottleDlg.h"
#include <conio.h>
#include "acpi.h"
#include "RegistryKey.hpp"
#include "OSdetect.h"
#include "gpioctl.h" 
#include <winioctl.h>



#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define DEFAULT_MIN_THRESH 15
#define DEFAULT_MAX_THRESH 30
#define DEFAULT_FS_THRESH  70
#define DEFAULT_MIN_THROTTLE 0
#define DEFAULT_MAX_THROTTLE 75
#define DEFAULT_SECONDS	5
#define DEFAULT_AUTO 0

using namespace JetByteTools;
// CThrottleApp

BEGIN_MESSAGE_MAP(CThrottleApp, CWinApp)
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()


// CThrottleApp construction

CThrottleApp::CThrottleApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CThrottleApp object

CThrottleApp theApp;

	// make some globals.
	WORD g_acpiBaseAddress=0xcafe;			// default to a known, wack value
	BYTE g_newThrottleValue=0x42;			// the ultimate answer.  
	DWORD g_minCPU = DEFAULT_MIN_THRESH;	// using DWORDs so sscanf doesn't overwrite any data
	DWORD g_maxCPU = DEFAULT_MAX_THRESH;
	DWORD g_fsCPU  = DEFAULT_FS_THRESH;
	DWORD g_minThrottle = DEFAULT_MIN_THROTTLE;
	DWORD g_maxThrottle = DEFAULT_MAX_THROTTLE;
	DWORD g_delay = DEFAULT_SECONDS;
	DWORD g_autoenabled = FALSE;				// flag to check the auto-throttle enable box in the dialog.
	DWORD g_initializing = 1;

	
void LoadSettings()
{
	// open the key
	HKEY key;
    if (ERROR_SUCCESS != RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\WinThrottle", 0, KEY_READ, &key))
		return;
    
	// read the values
    unsigned long len;
    DWORD type, val;
	
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "min_thresh", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_minCPU = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "max_thresh", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_maxCPU = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "fs_thresh", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_fsCPU = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "delay", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_delay = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "auto", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_autoenabled = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "max_throttle", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_maxThrottle = val;
	len = sizeof(val);
    if (ERROR_SUCCESS == RegQueryValueEx(key, "min_throttle", NULL, &type, (LPBYTE)&val, &len) && type == REG_DWORD)
		g_minThrottle = val;
        
	// close the key
	RegCloseKey(key);
}
	
void SaveSettings()
{
	// don't bother saving if these values are already there.
	static DWORD s_prev_minCPU=0xFFFFFFFF, s_prev_maxCPU=0xFFFFFFFF, s_prev_fs_thresh=0xFFFFFFFF, s_prev_delay=0xFFFFFFFF, s_prev_auto=0xFFFFFFFF, s_prev_minThrottle=0xFFFFFFFF, s_prev_maxThrottle=0xFFFFFFFF;
	if (s_prev_minCPU==g_minCPU && 
		s_prev_maxCPU==g_maxCPU && 
		s_prev_fs_thresh==g_fsCPU &&
		s_prev_delay ==g_delay  &&
		s_prev_auto  ==g_autoenabled   && 
		s_prev_minThrottle == g_minThrottle && 
		s_prev_maxThrottle == g_maxThrottle)
	{
		return;
	}
	else {
		s_prev_minCPU = g_minCPU;
		s_prev_maxCPU = g_maxCPU;
		s_prev_fs_thresh = g_fsCPU;
		s_prev_delay  = g_delay;
		s_prev_auto   = g_autoenabled;
		s_prev_minThrottle = g_minThrottle;
		s_prev_maxThrottle = g_maxThrottle;
	}

	// try to create the key (or just opens it, if it already existed)
    HKEY key;
	if (ERROR_SUCCESS != RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\WinThrottle", 0, NULL, 0, KEY_CREATE_SUB_KEY|KEY_WRITE, NULL, &key, NULL))
		return;

	// write the values to the key
	DWORD val;
	val = g_minCPU; RegSetValueEx(key, "min_thresh", NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
    val = g_maxCPU; RegSetValueEx(key, "max_thresh", NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
	val = g_fsCPU;  RegSetValueEx(key, "fs_thresh" , NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
    val = g_delay;  RegSetValueEx(key, "delay"     , NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
    val = g_autoenabled;   RegSetValueEx(key, "auto"      , NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
	val = g_minThrottle; RegSetValueEx(key, "min_throttle", NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
    val = g_maxThrottle; RegSetValueEx(key, "max_throttle", NULL, REG_DWORD, (const BYTE *)&val, sizeof(DWORD));
    
	// close the key
	RegCloseKey(key);
}
	
	


// CThrottleApp initialization

BOOL CThrottleApp::InitInstance()
{
	// check for another instance running; if found, give it the focus, and exit.
	HWND hWndOtherInstance = ::FindWindow("#32770", "winThrottle");
	if (hWndOtherInstance) {
		::ShowWindow(hWndOtherInstance, SW_SHOW);
		::SetForegroundWindow(hWndOtherInstance);
		::SetActiveWindow(hWndOtherInstance);
		exit(0);
	}

	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	LoadSettings();

	CWinApp::InitInstance();

	// check for a command line value to set throttle value to automagically.
	// we can have either of the following command lines:
	// winThrottle x
	//    Which will just take a static value from 0-15 and drop it into the throttle mechanism and exit.
	// or
	// 
	// winThrottle min=5 max=20 delay=2
	//    Which will set the auto-throttling mechanism with the supplied defaults and minimize it. 

	CString cmdLine;
	CString tmp;
	int pos;

	cmdLine=GetCommandLine();
	pos = cmdLine.Find("=");
	// grab min, max, delay values from command line.  Use defaults if any values are missing.
	if (pos !=0xffffffff)
		{
			pos = cmdLine.Find("min=");		
			if (pos != 0xffffffff)
			{
				tmp=cmdLine.Mid(pos+4,2);
				sscanf(tmp,"%d", &g_minCPU);
				g_autoenabled = TRUE;
			}

			pos = cmdLine.Find("max=");
			if (pos != 0xffffffff)
			{
				tmp=cmdLine.Mid(pos+4,2);
				sscanf(tmp,"%d", &g_maxCPU);
				g_autoenabled = TRUE;
			}

			pos = cmdLine.Find("delay=");
			if (pos != 0xffffffff)
			{				
				tmp=cmdLine.Mid(pos+6,3);
				sscanf(tmp,"%d", &g_delay);
				g_autoenabled = TRUE;
			}	

			pos = cmdLine.Find("fs=");
			if (pos != 0xffffffff)
			{
				tmp=cmdLine.Mid(pos+3,2);
				sscanf(tmp,"%d", &g_fsCPU);
				g_autoenabled = TRUE;
			}
		}
		

	// otherwise, there may just be a number to jam in.  Use the old method to figure that out.
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);
	sscanf(cmdInfo.m_strFileName,"%d",&g_newThrottleValue);

	if ((g_newThrottleValue > 15) && (g_newThrottleValue != 0x42))
	{
		AfxMessageBox("Invalid throttle value on command line.  Needs to be 0-15\r\n0=Off, 15=Slowest", MB_ICONSTOP, 0);
		return (false);		// quit.
	}

	DWORD viaVal=0xff;	// default to nothing

	// open the throttle driver
	if (! openIO())
	{
		AfxMessageBox("Unable to open or locate the throttle.sys driver.", MB_ICONSTOP, 0);
		return (false);		// quit.
	}
	

	// detect a known chipset
	detectChipset();


	// launch the slider dialog if we've disovered a good chipset.
	if (g_acpiBaseAddress == 0xcafe)
	{
			AfxMessageBox("Sorry, unable to get ACPI info from registry.", MB_ICONSTOP, 0);
			return (false);  
	}

	// detect a VIA chipset by seeing if bit 0 in the throttle register is writable.
	// also bail out if we got nuthin back from the ACPI I/O space.
	viaVal = isVia();
		
	if (viaVal == 0xff)
		{
			AfxMessageBox("Sorry, ACPI I/O space is unallocated.", MB_ICONSTOP, 0);
			return (false);
		}

	if (viaVal == 0xfe)
		{
			AfxMessageBox("Sorry, it appears the throttle driver isn't working.", MB_ICONSTOP, 0);
			return (false);
		}


	// ok, we've jumped through all the hoops. 
	
	// if we've got a command line argument, skip the dialog and just write it in.
	if (g_newThrottleValue !=0x42)
	{
		// convert input value 0-15 to a new throttle value
		if (viaVal)
			g_newThrottleValue=(16 - g_newThrottleValue) | THROTTLE_ENABLE_BIT;
		else
		{	
			g_newThrottleValue = (g_newThrottleValue & 0xfe);  // clear bit 0 as that bit is read only.
			g_newThrottleValue=(16-g_newThrottleValue) | THROTTLE_ENABLE_BIT;
		}
		if (g_newThrottleValue==THROTTLE_ENABLE_BIT) g_newThrottleValue=0;  // enable but no value? Turn off.
		ioPortWriteByte(g_acpiBaseAddress+THROTTLE_REGISTER, g_newThrottleValue); //_outp(g_acpiBaseAddress+THROTTLE_REGISTER, g_newThrottleValue);
		return (false);
	}

	// launch a dialog to get the new throttle value
	CThrottleDlg dlg;
	m_pMainWnd = &dlg;

	// pass min, max and delay values to the dialog
	dlg.minCPU = (BYTE)g_minCPU; 
	dlg.maxCPU = (BYTE)g_maxCPU;
	dlg.fsCPU  = (BYTE)g_fsCPU;
	dlg.minThrottle = (int)g_minThrottle;
	dlg.maxThrottle = (int)g_maxThrottle;
	dlg.delay = (BYTE)g_delay;
	dlg.autoEnabled = g_autoenabled;
	dlg.minimizeAfterNSeconds = g_autoenabled ? 4 : -1; //firstTimeHide;

	dlg.isVia=(BOOL)viaVal;											// send a flag to our dialog if a via chipset is found
	g_newThrottleValue= ioPortReadByte(g_acpiBaseAddress+THROTTLE_REGISTER); //_inp(g_acpiBaseAddress+THROTTLE_REGISTER);	// read the current throttle value from the chipset
	dlg.ThrottleValue=g_newThrottleValue;							// pass the value into the dialog
	dlg.AcpiBaseAddress=g_acpiBaseAddress;							// pass the ACPI base addy in too.
	INT_PTR nResponse = dlg.DoModal();								// do dialog
		
	g_newThrottleValue=dlg.ThrottleValue;
	if (nResponse != IDOK)			// OK=keep throttle value and exit otherwise
		ioPortWriteByte(g_acpiBaseAddress+THROTTLE_REGISTER, 0); //_outp(g_acpiBaseAddress+THROTTLE_REGISTER, 0);				// disable all throttling, user has closed the app.

	CloseHandle(h);
	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return (false);
}

// returns 1 if a via board is detected, 0 if not, and 0xff if the ACPI I/O space is completely hosed up.
// VIA boards utilize bit 0 of the ACPI throttle register, allowing for 6.25% increments in throttling.
// Here we see if bit 0 is writable, and if it is, it's a pretty good indication that it's a VIA board.
DWORD CThrottleApp::isVia()
{
		BYTE tmpValue, savedValue;

		tmpValue=ioPortReadByte(g_acpiBaseAddress+THROTTLE_REGISTER); //_inp(g_acpiBaseAddress+THROTTLE_REGISTER);  // get the current value
		
		if ( (tmpValue == 0xff) || (tmpValue == 0xfe) )  // if port error, it'll return 0xff.  If driver error, we'll get 0xfe
			return (tmpValue);

		savedValue=tmpValue;								// save it away
		tmpValue |=1;										// set bit 0
		ioPortWriteByte(g_acpiBaseAddress+THROTTLE_REGISTER, tmpValue); //_outp(g_acpiBaseAddress+THROTTLE_REGISTER, tmpValue);
		tmpValue=ioPortReadByte(g_acpiBaseAddress+THROTTLE_REGISTER); //_inp(g_acpiBaseAddress+THROTTLE_REGISTER);  // read it back
		ioPortWriteByte(g_acpiBaseAddress+THROTTLE_REGISTER, savedValue); //_outp(g_acpiBaseAddress+THROTTLE_REGISTER, savedValue);		// restore old value back, just in case.
		
		return (tmpValue & 1);				// returns 1 if VIA, 0 if not.
}			

// return 0=win3.1 (yikes!)
// 1=98,95,me
// 2=NT/XP/2K
DWORD CThrottleApp::detectOS()
{
	OSVERSIONINFO OSversion;
	
	OSversion.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);

	::GetVersionEx(&OSversion);

	return(OSversion.dwPlatformId);
}


bool CThrottleApp::openIO()
{
	if (detectOS() < 2)
	{
		AfxMessageBox("Sorry, winThrottle requires Windows 2000 or higher.", MB_ICONSTOP, 0);
		return (false);
	}
	else
	{
	h = CreateFile("\\\\.\\throttle",
    	GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_EXISTING,
		0,
		NULL
		);

    if(h == INVALID_HANDLE_VALUE) 
           return (false);
    
	}
	return (true);
}

void CThrottleApp::detectChipset()
{

	/* the ACPI FACP table (renamed by windows to be the FADT table) is tucked away in the registry.
	   we need to read the contents of the FADT table to find the PM1 Event port address, which is where the ACPI I/O base
	   address is.  This is MUCH easier than scanning through the PCI devices looking for known hardware and should be
	   universal on all ACPI chipsets.

	   The FADT looks like this in the registry, at least on my machine:

		HKEY_LOCAL_MACHINE\HARDWARE\ACPI\FADT
		                                     \DELL__
		                                            \GX270__
		                                                    \00000007
		and this 00000007 key contains:
		46,41,43,50,74,00,00,00,01,8e,44,45,4c,4c,20,20,47,58,32,37,30,
		20,20,00,07,00,00,00,41,53,4c,20,61,00,00,00,00,00,e7,3f,16,32,fd,ff,01,00,
		09,00,b2,00,00,00,70,71,00,00,00,08,00,00,00,00,00,00,04,08,00,00,00,00,00,
		00,00,00,00,00,08,08,00,00,28,08,00,00,00,00,00,00,04,02,00,04,08,00,00,00,
		f4,01,88,13,00,00,00,00,00,00,00,00,00,00,00,00,a5,00,00,00

		and somewhere in that binary field of data is the base address port we're looking for.
		It's at offset 56 or 0x38 and is 4 bytes long.

	I have NO FREEKIN idea how this registry stuff works.  Change it if you dare.

*/
	CString FADTstring="HARDWARE\\ACPI\\FADT";
	CString keyval;    
	CString facpTable="";
	CString baseAddress;
	DWORD baseAddr;

	// open the FADT key	
	CRegistryKey key = CRegistryKey(HKEY_LOCAL_MACHINE, _T(FADTstring));

	// get the computer manufacturer key (in my case DELL)
	CRegistryKey::SubkeyIterator it = key.BeginSubkeyIteration();	
	CRegistryKey thisKey = it.OpenKey();
	keyval = it.GetName();

	FADTstring=FADTstring+"\\"+keyval;
	
	// open the next key down and get the model number (in my case GX270)
	key = CRegistryKey(HKEY_LOCAL_MACHINE, _T(FADTstring));
	it = key.BeginSubkeyIteration();	
	thisKey = it.OpenKey();
	keyval = it.GetName();

	FADTstring=FADTstring+"\\"+keyval;



	// open the final key where the data is
      key = CRegistryKey(HKEY_LOCAL_MACHINE, _T(FADTstring));

	  // no do some voodoo class nightmare stuff to get the data.
      for (CRegistryKey::SubkeyIterator it = key.BeginSubkeyIteration();
           it != key.EndSubkeyIteration();
           ++it)
      {
      
		CRegistryKey thisKey = it.OpenKey();
			
	   for (CRegistryKey::ValueIterator val = thisKey.BeginValueIteration();
              val != thisKey.EndValueIteration();
              val++)
         {
			 facpTable=facpTable+val.AsString();  
			 
         }
      }

	// now parse through the string of FACP data and see if any of it makes sense.
	// 'FACP' in this string is represented like this: '46414350' cuz its a string of ascii values. hehe.
	if (facpTable.Left(8) == "46414350")	// hey we grabbed the right data!
	{
		baseAddress=facpTable.Mid(56*2,8);		// grab the base address from this table.	  
		// now we need to big endianify this value
		baseAddress=baseAddress.Mid(6,2)+baseAddress.Mid(4,2)+baseAddress.Mid(2,2)+baseAddress.Mid(0,2);
		sscanf(baseAddress, "%X", &baseAddr);
		g_acpiBaseAddress=(WORD)baseAddr;	
	}
}

// read a byte from I/O space at portNum.  Portnum is 0-65535
BYTE CThrottleApp::ioPortReadByte(WORD portNum)
{

    ULONG   PortNumber;         // Buffer sent to the driver (Port #).
    ULONG   DataLength;
    DWORD   ReturnedLength;     // Number of bytes returned

	PortNumber = (ULONG)portNum;
	DataLength = sizeof(DataBuffer.CharData);
	BOOL IoctlResult = DeviceIoControl(
                            h,				    // Handle to device
                            IOCTL_GPD_READ_PORT_UCHAR,  // IO Control code for Read
                            &PortNumber,        // Buffer to driver.
                            sizeof(PortNumber), // Length of buffer in bytes.
                            &DataBuffer,        // Buffer from driver.
                            DataLength,         // Length of buffer in bytes.
                            &ReturnedLength,    // Bytes placed in DataBuffer.
                            NULL                // NULL means wait till op. completes.
                            );
	if (IoctlResult != TRUE)
			DataBuffer.CharData = 0xfe;			// if read fails, return crap data

	return (DataBuffer.CharData);

}
void CThrottleApp::ioPortWriteByte(WORD portNum, BYTE val)
{

	GENPORT_WRITE_INPUT InputBuffer;    // Input buffer for DeviceIoControl
	InputBuffer.CharData = (UCHAR)val;
	InputBuffer.PortNumber = (ULONG)portNum;
	ULONG				DataLength = offsetof(GENPORT_WRITE_INPUT, CharData) + sizeof(InputBuffer.CharData);
    ULONG               ReturnedLength; // Number of bytes returned in output buffer

	BOOL  IoctlResult = DeviceIoControl(
                        h,                // Handle to device
                        IOCTL_GPD_WRITE_PORT_UCHAR, // IO Control code for Write
                        &InputBuffer,           // Buffer to driver.  Holds port & data.
                        DataLength,             // Length of buffer in bytes.
                        NULL,                   // Buffer from driver.   Not used.
                        0,                      // Length of buffer in bytes.
                        &ReturnedLength,        // Bytes placed in outbuf.  Should be 0.
                        NULL                    // NULL means wait till I/O completes.
                        );
}
