#include "stdafx.h"

#include "TrayIcon.h"

CTrayIcon::CTrayIcon(HWND hWnd, UINT uIconID, HICON hIcon, LPSTR lpToolTip, MouseMsgHandlerPtr *pMouseMsgHandler, int nHandlers)
{
	SetTrayIcon(hWnd, uIconID, hIcon, lpToolTip);

	SetMouseMsgHandler(pMouseMsgHandler, nHandlers);
}

CTrayIcon::~CTrayIcon()
{
	DeleteIcon();
//	for( int i = 0 ; i < m_nHandlers ; ++i )
//	    delete m_pMouseMsgHandler[i];

	delete m_pMouseMsgHandler;
}

void CTrayIcon::SetTrayIcon(HWND hWnd, UINT uIconID, HICON hIcon, LPSTR lpToolTip)
{
	m_hWnd = hWnd;
	
	m_uIconID = uIconID;
	
	m_hIcon = hIcon;
	
	if (lpToolTip)
		lstrcpyn(m_szToolTipText, lpToolTip, sizeof(m_szToolTipText));
	else
		m_szToolTipText[0] = 0;
}

void CTrayIcon::setToolTip(LPSTR lpToolTip)
{
	if (lpToolTip)
		lstrcpyn(m_szToolTipText, lpToolTip, sizeof(m_szToolTipText));
	else
		m_szToolTipText[0] = 0;
}

void CTrayIcon::SetMouseMsgHandler(MouseMsgHandlerPtr *pMouseMsgHandler, int nHandlers)
{
	m_pMouseMsgHandler = pMouseMsgHandler;

	m_nHandlers = nHandlers;
}

LRESULT CTrayIcon::OnTaskBarCreated(WPARAM wParam, LPARAM lParam)
{
	VERIFY(AddIcon());
    return 0;
}

void CTrayIcon::OnNotifyIcon(WPARAM wParam, LPARAM lParam)
{
	UINT uID; 
    UINT uMouseMsg; 
 
    uID = (UINT) wParam; 
    uMouseMsg = (UINT) lParam; 
 
	for(int i=0; i<m_nHandlers; i++)
		if(uID == m_uIconID)
			m_pMouseMsgHandler[i]->MouseMsgHandler(uMouseMsg, wParam, lParam);	
}

BOOL CTrayIcon::AddIcon()
{
	BOOL res; 
    NOTIFYICONDATA tnid; 
 
    tnid.cbSize = sizeof(NOTIFYICONDATA); 
    tnid.hWnd = m_hWnd; 
    tnid.uID = m_uIconID; 
    tnid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; 
    tnid.uCallbackMessage = WM_TI_NOTIFYICON; 
    tnid.hIcon = m_hIcon; 
    lstrcpyn(tnid.szTip, m_szToolTipText, sizeof(tnid.szTip)); 
 
    res = Shell_NotifyIcon(NIM_ADD, &tnid); 
 
    if (m_hIcon) 
        DestroyIcon(m_hIcon); 
 
    return res; 
}
BOOL CTrayIcon::upDateIcon()
{
	BOOL res; 
    NOTIFYICONDATA tnid; 
 
    tnid.cbSize = sizeof(NOTIFYICONDATA); 
    tnid.hWnd = m_hWnd; 
    tnid.uID = m_uIconID; 
    tnid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; 
    tnid.uCallbackMessage = WM_TI_NOTIFYICON; 
    tnid.hIcon = m_hIcon; 
    lstrcpyn(tnid.szTip, m_szToolTipText, sizeof(tnid.szTip)); 
 
    res = Shell_NotifyIcon(NIM_MODIFY, &tnid); 
 
    if (m_hIcon) 
        DestroyIcon(m_hIcon); 
 
    return res; 
}
BOOL CTrayIcon::DeleteIcon()
{
	BOOL res; 
    NOTIFYICONDATA tnid; 
 
    tnid.cbSize = sizeof(NOTIFYICONDATA); 
    tnid.hWnd = m_hWnd; 
    tnid.uID = m_uIconID; 
         
    res = Shell_NotifyIcon(NIM_DELETE, &tnid); 
    return res; 
}